/**
 * (c) 2008 REINER SCT
 *
 * $Date: 2010-09-29 19:11:23 +0200 (Mi, 29. Sep 2010) $
 * $Rev: 1476 $
 */

var opttanObject    = null;

function JSOpttan(input, device, delay) {

    if (!device) alert('No device given');

    this.delay = delay ? delay : 0;  // milliseconds to next step
    this.device = device;

    var datapos = 0;
    var timerLock = false;

    var high = 1;
    var low = 0;

    var vtab = new Array();
    vtab['0'] = [low ,low ,low ,low ];
    vtab['1'] = [low ,low ,low ,high];
    vtab['2'] = [low ,low ,high,low ];
    vtab['3'] = [low ,low ,high,high];
    vtab['4'] = [low ,high,low ,low ];
    vtab['5'] = [low ,high,low ,high];
    vtab['6'] = [low ,high,high,low ];
    vtab['7'] = [low ,high,high,high];
    vtab['8'] = [high,low ,low ,low ];
    vtab['9'] = [high,low ,low ,high];
    vtab['A'] = [high,low ,high,low ];
    vtab['B'] = [high,low ,high,high];
    vtab['C'] = [high,high,low ,low ];
    vtab['D'] = [high,high,low ,high];
    vtab['E'] = [high,high,high,low ];
    vtab['F'] = [high,high,high,high];

    var ST_CLK1_UP = 0;
    var ST_CLK1_DN = 1;

    var usPos = 0;
    var usState = ST_CLK1_UP;


    this.setOpttan = function (clk1, nibble) {
      this.displayField('rsct_' + this.device + "_clk1", clk1);
      if (nibble) {
        this.displayField('rsct_' + this.device + "_data0", vtab[nibble][3]);
        this.displayField('rsct_' + this.device + "_data1", vtab[nibble][2]);
        this.displayField('rsct_' + this.device + "_data2", vtab[nibble][1]);
        this.displayField('rsct_' + this.device + "_data3", vtab[nibble][0]);
      }
    };

    this.displayField = function (fld, show) {
      document.getElementById(fld).style.visibility = show ? 'visible' : 'hidden';
    };

    this.nextTimer = function () {

      if (timerLock)
        return;
      timerLock = true;

      switch (usState) {
        case ST_CLK1_UP :
          this.setOpttan(high, data[usPos]);
          usState = ST_CLK1_DN;
          break;
        case ST_CLK1_DN :
          this.setOpttan(low , data[usPos]);
          usState = ST_CLK1_UP;
          usPos++;
          if (usPos >= data.length)
            usPos = 0;
          break;
      }

      timerLock = false;
    };

    this.sortBytes = function (input) {
      var new_data = "";
      for (var i=0; i< input.length; i=i+2) {
        new_data = new_data + input.substr(i+1, 1) + input.substr(i, 1);
      }
      return new_data;
    };

    this.html = function () {

      var path_prefix = opttanConfig.path + opttanConfig.imgPath + "rsct_" + this.device;
      var content = "<img id='rsct_" + this.device + "_clk1'  style='width:100%; height:100%;' src='" + path_prefix + "_clk1.gif' alt='' />"
                  + "<img id='rsct_" + this.device + "_data0' style='width:100%; height:100%;' src='" + path_prefix + "_data0.gif' alt='' />"
                  + "<img id='rsct_" + this.device + "_data1' style='width:100%; height:100%;' src='" + path_prefix + "_data1.gif' alt='' />"
                  + "<img id='rsct_" + this.device + "_data2' style='width:100%; height:100%;' src='" + path_prefix + "_data2.gif' alt='' />"
                  + "<img id='rsct_" + this.device + "_data3' style='width:100%; height:100%;' src='" + path_prefix + "_data3.gif' alt='' />"
                  + "<img id='rsct_" + this.device + "_fg'    style='width:100%; height:100%;' src='" + path_prefix + "_fg.gif' alt='' usemap='#" + this.device + "_map' border='0' />"
                  + "<span class='opttan_speedo' id='rsct_" + this.device + "_speedo' >100%</span>";

      var mapcoords = new Array();
      mapcoords['opttan_m_minus'] = "0,0,42,95";
      mapcoords['opttan_m_plus'] = "164,0,200,95";
      mapcoords['opttan_mr_minus'] = "0,0,95,42";
      mapcoords['opttan_mr_plus'] = "0,164,95,200";

      content += "<map name='" + this.device + "_map'>"
              + "<area shape='rect' id='" + this.device + "_map_minus' coords='" + mapcoords[this.device + "_minus" ] + "' href='javascript:opttanObject.setDelay(1)' alt='' />"
              + "<area shape='rect' id='" + this.device + "_map_plus'  coords='" + mapcoords[this.device + "_plus" ] + "' href='javascript:opttanObject.setDelay(-1)' alt='' />"
              + "</map>";

      return content;
    };

    this.start = function () {
      var thisC = this;
      this.stop();
      this.interval = window.setInterval(function () { thisC.nextTimer(); }, 20 + this.delay * 2);
    };

    this.stop = function () {
      if (this.interval)
        window.clearInterval(this.interval);
    };

    this.showSpeedo = function () {
      var thisC = this;
      if (this.speedo_timer)
        window.clearTimeout(this.speedo_timer);
      thisC.speedo = document.getElementById('rsct_' + this.device + '_speedo');
      thisC.speedo.innerHTML = 100 -  this.delay + '%';
      thisC.speedo.style.visibility = 'visible';
      this.speedo_timer = window.setTimeout(function () { thisC.speedo.style.visibility = 'hidden'; }, 2000);
    };

    this.setDelay = function (value) {
      var newdelay = parseInt(this.delay) + value*5;
      if (newdelay >= 0 && newdelay <= 100) {
        this.delay = newdelay;
        this.start();
        this.showSpeedo();
      }
      return this.delay;
    };

    var data = "0FF"; // SYNC
    data = (data + this.sortBytes(input).toUpperCase()).split('');
}

function GifOpttan(data, device) {
    if (!device) alert('No device given');

    this.fps = 15; // max possible framerate for IE
    this.data = data;
    this.device = device;

    this.html = function () {

      if (!opttanConfig.gifGeneratorPath) {
        var content = "<div id='rsct_app'>";
        content += "Fehler: Der Pfad zum Gif-Generator ist nicht gesetzt (Konfiguration: gifGeneratorPath).";
        content += "</div>";
          return content;
      }

      var content = "<img src='" + opttanConfig.gifGeneratorPath + "?data=" + this.data + "&fps=" + this.fps + "&device=" + this.device + "' id='rsct_" + this.device + "_gif' alt='' style='width:100%; height:100%;' />";
      if (this.device == 'opttan') {
        content += "<div><img id='rsct_opttan_fg_image' src='" + opttanConfig.path + opttanConfig.imgPath + "rsct_opttan_fg.gif' alt='' style='width:100%; height:100%;' /></div>";
      }
      return content;
    };

}

function FlashOpttan(data, device, delay) {
    if (!device) alert('No device given');

    this.delay = delay ? delay : 0;  // milliseconds to next step
    this.data = data;
    this.device = device;

    this.html = function () {
      var swf_file = opttanConfig.path + opttanConfig.flashPath + "rsct_" + this.device + ".swf";
      var content = "<object type='application/x-shockwave-flash' data='" + swf_file + "' style='width:100%; height:100%;' id='rsct_" + this.device + "_swf'>"
                  + "<param name='movie' value='" + swf_file + "' />"
                  + "<param name='FlashVars' value='data=" + this.data + "&delay=" + this.delay + "' />"
                  + "</object>";
      return content;
    };

    this.setDelay = function (value) {
      var movie = document["rsct_" + this.device + "_swf"];
      if (movie.PercentLoaded) {
        if (movie.PercentLoaded() == 100) {
          movie.SetVariable("_root.change_delay", value );
          return movie.GetVariable("_root.delay");
        }
      }
      return 0;
    };
}

function showOpttan(data, opttan_path, type) {
    opttanObject = new Opttan(data, opttan_path, type);
    document.write(opttanObject.display());
    this.timeout = window.setTimeout(function () { opttanObject.setup(); }, 100);
}

function Opttan(data, opttan_path, type) {
    opttanConfig.path = opttan_path;
    opttanConfig.imgPath = opttanConfig.imgPath ? opttanConfig.imgPath : 'images/';
    opttanConfig.flashPath = opttanConfig.flashPath ? opttanConfig.flashPath : 'flash/';
    this.cookie_prefix = opttanConfig.cookiePrefix;
    var selected_type = false;
    type = type ? type : 0;
    this.types = ['auto', 'flash', 'js', 'gif'];
    this.apps = new Array();
    this.settings = new Array();

    this.toggle = function (device) {
      var area = "rsct_" + device + "_area";
      var visibility = window.document.getElementById(area).style.display;
      if (visibility == '' || visibility == 'visible') {
        this.fold(device);
      } else {
        this.unfold(device);
      }
    };

    this.stop = function (device) {
      for (app in this.apps) {
        if (this.apps[app].device == device && this.apps[app].stop) {
          this.apps[app].stop();
        }
      }
    };

    this.start = function (device) {
      for (app in this.apps) {
        if (this.apps[app].device == device && this.apps[app].start) {
          this.apps[app].start();
        }
      }
    };

    this.fold = function (device) {
      var area = "rsct_" + device + "_area";
      var btn  = "rsct_" + device + "_show_btn";
      var btns = "rsct_" + device + "_left_buttons";

      this.stop(device);

      document.getElementById(area).style.display = "none";
      document.getElementById(btns).style.display = "none";
      document.getElementById(btn).src = opttanConfig.path + opttanConfig.imgPath + "rsct_fold.gif";
      this.setCookie('show_' + device, 0);
    };

    this.unfold = function (device) {
      var area = "rsct_" + device + "_area";
      var btn  = "rsct_" + device + "_show_btn";
      var btns = "rsct_" + device + "_left_buttons";

      this.start(device);

      document.getElementById(area).style.display = "";
      document.getElementById(btns).style.display = "";
      document.getElementById(btn).src = opttanConfig.path + opttanConfig.imgPath + "rsct_unfold.gif";
      this.setCookie('show_' + device, 1);
    };

    this.resizeWH = function (newwidth, device) {

      if (!newwidth) return;
      if (device == 'opttan_m' && newwidth < 120) return;
      if (device == 'opttan_mr' && newwidth < 80) return;
      if (!document.getElementById('rsct_' + device)) return;

      var oldwidth  = this.getWidth(device);
      var oldheight = this.getHeight(device);
      var oldappw   = this.getWidth(device + '_app');
      var oldapph   = this.getHeight(device + '_app');

      var fac = (newwidth*1.0/oldwidth*1.0);

      this.setCookie('width_' + device, Math.round(newwidth));

      var unit = 'px';
      var h = oldheight * fac;
      var appw = oldappw * fac;
      var apph = oldapph * fac;

      var framesizes = ['rsct_' + device, 'rsct_' + device + '_bg_image'];
      for (i in framesizes) {
        var elem = document.getElementById(framesizes[i]);
        if (elem) {
          elem.style.height = h + unit;
          elem.style.width  = newwidth + unit;
        }
      }

      var app = document.getElementById('rsct_' + device + '_app');
      if (app) {
        app.style.height = apph + unit;
        app.style.width  = appw + unit;
        app.style.top = ((h - apph)/2) + unit;
        app.style.left = ((newwidth - appw)/2) + unit;
      }

      var font_size = Math.round(newwidth / 8);
      var speedo   = document.getElementById('rsct_' + device + '_speedo');
      var minusmap = document.getElementById(device + '_map_minus');
      var plusmap  = document.getElementById(device + '_map_plus');
      switch (device) {
        case 'opttan_m':
          if (minusmap) minusmap.setAttribute('coords',"0,0," + Math.round(appw/5) + "," + apph);
          if (plusmap)  plusmap.setAttribute('coords', Math.round(appw - appw/5) + ",0," + appw + "," + apph);
          if (speedo)   speedo.style.top = Math.round(apph/2 - font_size/2) + unit;
          break;
        case 'opttan_mr':
          if (minusmap) minusmap.setAttribute('coords', "0," + Math.round(apph - apph/5) + ',' + appw + "," + apph);
          if (plusmap)  plusmap.setAttribute('coords',"0,0," + appw + "," + Math.round(apph/5));
          if (speedo)   speedo.style.top = Math.round(apph/2 - font_size/2) + unit;
          break;
      }
      if (speedo) speedo.style.fontSize = font_size + unit;
    };

    this.saveCookie = function () {
      var old_prefix = this.cookie_prefix;
      this.cookie_prefix = opttanConfig.cookiePrefix;
      this.writeSettings();
      this.cookie_prefix = old_prefix;
    };

    this.resetCookie = function () {
      this.setCookie('width', '');
    };

    this.getWidth = function (device) {
      var o = document.getElementById('rsct_' + device);
      return o ? o.offsetWidth : 0;
    };

    this.getHeight = function (device) {
      var o = document.getElementById('rsct_' + device);
      return o ? o.offsetHeight : 0;
    };

    this.resize = function (step, device) {
      var width = Math.round(this.getWidth(device));
      if (width > 0) this.resizeWH(width  * (1 + step/20), device);
      // #108 bugfix IE6
      document.body.className = document.body.className;
    };

    this.customize = function (device) {
      var img = new Image();
      var o = document.getElementById('rsct_'+device+'_bg_image');
      if (!o) return;
      o.onload = function () {};
      img.onload = function () {
        o.src = img.src;
      };
      img.src = opttanConfig.path + (opttanConfig.customPath ? opttanConfig.customPath : 'custom_images') + "/rsct_" + device + "_bg.gif";
    };

    this.setup = function () {
      if (document.getElementById('rsct_opttan_bg_image') && !document.getElementById('rsct_opttan_bg_image').complete) {
        document.getElementById('rsct_opttan_bg_image').src = opttanConfig.path + opttanConfig.imgPath + "rsct_opttan_bg.gif";
      }
      if (document.getElementById('rsct_opttan_m_bg_image') && !document.getElementById('rsct_opttan_m_bg_image').complete) {
        document.getElementById('rsct_opttan_m_bg_image').src = opttanConfig.path + opttanConfig.imgPath + "rsct_opttan_m_bg.gif";
      }
      if (document.getElementById('rsct_opttan_mr_bg_image') && !document.getElementById('rsct_opttan_mr_bg_image').complete) {
        document.getElementById('rsct_opttan_mr_bg_image').src = opttanConfig.path + opttanConfig.imgPath + "rsct_opttan_mr_bg.gif";
      }
      if (opttanConfig.showOpttan) {
        this.showDevice('opttan', opttanConfig.defaultSize);
      }
      if (opttanConfig.showOpttanMini) {
        var dev = this.getCookieValue('rotate') == '1' ? 'opttan_mr' : 'opttan_m';
        this.showDevice(dev, opttanConfig.defaultSize);
      }

      var show = this.getCookieValue('show_opttan_m');
      if (show == 0 || (show == null && opttanConfig.foldOpttanMini)) this.fold('opttan_m');
      show = this.getCookieValue('show_opttan_mr');
      if (show == 0 || (show == null && opttanConfig.foldOpttanMini)) this.fold('opttan_mr');
    };

    this.setDelay = function (value) {
      for (app in this.apps) {
        if (this.apps[app].setDelay) {
          var newdelay = this.apps[app].setDelay(value);
          this.setCookie('delay', newdelay);
        }
      }
    };

    this.setCookie = function (key, value, ignore) {
      this.readSettings();
      this.settings[key] = value;
      this.writeSettings();
    };

    this.getCookieValue = function (key) {
      this.readSettings();
      if (this.settings[key])
        return(this.settings[key]);
      else
        return null;
    };

    this.readSettings = function () {
      var settings = '';
      if (opttanConfig.userSettingsId && document.getElementById(opttanConfig.userSettingsId)) {
        settings = unescape(document.getElementById(opttanConfig.userSettingsId).value);
      } else {
        var name = this.cookie_prefix  + "_data";
        var usersettings = typeof(document) != 'undefined' ? document.cookie : '';
        var found = usersettings.indexOf(name + "=");
        if (found > -1) {
          var start =  found + name.length + 1;
          var end = usersettings.indexOf('; ', start);
          if (end == -1) {
            end = usersettings.length;
          }
          settings = unescape(usersettings.substring(start, end));
        }
      }

      if (settings != "") {
        var temp = settings.split("&");
        for (var i=0; i < temp.length; i++) {
          var t2 = temp[i].split("=");
          this.settings[t2[0]] = t2[1];
        }
      }
    };

    this.writeSettings = function () {
      var strtmp = new Array();
      for (var key in this.settings) {
        if (key != "") strtmp.push(key + "=" + this.settings[key]);
      }
      if (opttanConfig.userSettingsId && document.getElementById(opttanConfig.userSettingsId)) {
        document.getElementById(opttanConfig.userSettingsId).value = escape(strtmp.join('&'));
      } else {
        var cookie_string = this.cookie_prefix + "_data=" + escape(strtmp.join('&'));
        if (opttanConfig.cookieExpire > 0) {
          var expires = new Date((new Date()).getTime() + 1000 * opttanConfig.cookieExpire);
          cookie_string += "; expires=" + expires.toGMTString();
        }
        cookie_string += "; path=/";
        document.cookie = cookie_string;
      }
    };

    this.displayDevice = function (device, content) {
      if (!device) alert('No device given');
      var app = selectApp(device, this.getDelay());
      var out = '';
      out += this.showTitlebar(device);
      out += "<tr id='rsct_" + device + "_area' style='display:none'>";
      out += "<td><div id='rsct_" + device + "'>";
      out += showBackground(device);
      out += "<div id='rsct_" + device + "_app'>";
      out += app.html();
      out += "</div>";
      out += "</div></td>";
      if (content)
        out += "<td>" + content + "</td>";
      out += "</tr>";
      this.apps.push(app);
      return out;
    };

    this.rotate = function (device) {

      switch (device) {
        case 'opttan_m':
          this.hideDevice('opttan_m');
          this.showDevice('opttan_mr');
          this.setCookie('rotate', 1);
          break;
        case 'opttan_mr':
          this.hideDevice('opttan_mr');
          this.showDevice('opttan_m');
          this.setCookie('rotate', 0);
          break;
      }

      for (app in this.apps) {
        if (this.apps[app].start) {
          this.apps[app].start();
        }
      }

    };

    this.hideDevice = function (device) {
      this.stop(device);
      document.getElementById('rsct_' + device + '_area').style.display = 'none';
      document.getElementById('rsct_' + device + '_titlebar').style.display = 'none';
    };

    this.showDevice = function (device, size) {
      if (!size) size = 1;
      document.getElementById('rsct_' + device + '_area').style.display = '';
      document.getElementById('rsct_' + device + '_titlebar').style.display = '';
      this.resizeWH((this.getCookieValue('width_' + device) ? this.getCookieValue('width_' + device) : this.getWidth(device) * size), device);
      this.start(device);
    };

    function selectApp(device, delay) {
      if (!device) alert('No device given');
      if (type > 0) {
        selected_type = type;
        if (type == 1) return new FlashOpttan(data, device, delay);
        if (type == 2) return new JSOpttan(data, device, delay);
        if (type == 3) return new GifOpttan(data, device);
        return 'wrong type';
      }
      if (isTerminalServer()) {
        selected_type = 3;
        return new GifOpttan(data, device);
      } else if (DetectFlashVer(4, 0, 0)) {
        selected_type = 1;
        return new FlashOpttan(data, device, delay);
      } else {
        selected_type = 2;
        return new JSOpttan(data, device, delay);
      }
    }

    this.getDelay = function () {
      return this.getCookieValue('delay') ? parseInt(this.getCookieValue('delay')) : opttanConfig.defaultDelay;
    };

    function isTerminalServer() {
      try {
        if (navigator.userAgent.indexOf("Windows NT 5.2") != -1) return true; //win 2003 server
        if (navigator.userAgent.indexOf("WTS") != -1) return true; //windows terminal server
      } catch(e) {
        // do nothing
      }

      try {
        var a = new ActiveXObject("RsctFt.RsctFtCtrl.1");
      } catch(e) {
        return false;
      }
      return true;
    };

    this.showTitlebar = function (device) {
      if (!device) alert('No device given');
      var header = '';
      switch (device) {
        case 'opttan_m':
          header = opttanConfig.opttanMiniHeader;
          break;
        case 'opttan_mr':
          header = opttanConfig.opttanMiniRHeader;
          break;
      }

      var content = '';
      content += "<tr id='rsct_" + device + "_titlebar' style='display:none'><td class='rsct_titlebar' ";
      content += "style='background: url(" + opttanConfig.path + opttanConfig.imgPath + "rsct_titlebar.gif) 0 0;' colspan='2'>";
      content += "<div class='rsct_right_buttons'>";
      content += "<a href=\"javascript:opttanObject.toggle('" + device + "')\">"
              + "<img id='rsct_" + device + "_show_btn' class='rsct_show_button' src='"
              + opttanConfig.path + opttanConfig.imgPath + "rsct_unfold.gif' alt='Ein / Ausblenden' title='Ein / Ausblenden' /></a>";
      content += "</div>";
      content += "<div class='rsct_left_buttons' id='rsct_" + device + "_left_buttons'>";
      content += "<a href=\"javascript:opttanObject.resize(1,'" + device + "');\"><img src='" + opttanConfig.path + opttanConfig.imgPath + "rsct_larger.png' alt='Grafik vergrern' title='Grafik vergrern' /></a>";
      content += "<a href=\"javascript:opttanObject.resize(-1,'" + device + "');\"><img src='" + opttanConfig.path + opttanConfig.imgPath + "rsct_smaller.png' alt='Grafik verkleinern' title='Grafik verkleinern' /></a>";
      if (opttanConfig.speedButtons && selected_type != 3) {
        content += "<a href=\"javascript:opttanObject.setDelay(1);\"><img src='" + opttanConfig.path + opttanConfig.imgPath + "rsct_slower.png' alt='Geschwindigkeit verkleinern' title='Geschwindigkeit verkleinern' /></a>";
        content += "<a href=\"javascript:opttanObject.setDelay(-1);\"><img src='" + opttanConfig.path + opttanConfig.imgPath + "rsct_faster.png' alt='Geschwindigkeit erhhen' title='Geschwindigkeit erhhen' /></a>";
      }
      if (device == 'opttan_m')
        content += "<a href=\"javascript:opttanObject.rotate('opttan_m');\"><img src='" + opttanConfig.path + opttanConfig.imgPath + "rsct_rotate_left.png' alt='Grafik drehen' title='Grafik drehen' /></a>";
      if (device == 'opttan_mr')
        content += "<a href=\"javascript:opttanObject.rotate('opttan_mr');\"><img src='" + opttanConfig.path + opttanConfig.imgPath + "rsct_rotate_right.png' alt='Grafik drehen' title='Grafik drehen' /></a>";
      content += "</div>&nbsp;";
      content += header;
      content += "</td></tr>";
      return content;
    };

    function showBackground(device) {
      return "<div><img id='rsct_" + device + "_bg_image' src='" + opttanConfig.path + opttanConfig.imgPath + "rsct_" + device + "_bg.gif' alt='' onload='opttanObject.customize(\""+device+"\")' /></div>";
    };

    this.showTable = function (content) {
      var head = "<table id='rsct_opttan_generation'>";
      var tail = "</table>";
      return head + content + tail;
    };

    this.tanHeader = function () {
      return "<tr><td class='rsct_titlebar' style='background: url(" + opttanConfig.path + opttanConfig.imgPath + "rsct_titlebar.gif) 0 0;' colspan='2'>&nbsp;TAN-Generierung mit:</td></tr>";
    };

    this.display = function () {
      var out = '';
      if (opttanConfig.showHeader)
        out += this.tanHeader();
      if (opttanConfig.showOpttanMini) {
        out += this.displayDevice("opttan_m", opttanConfig.opttanMiniContent);
        out += this.displayDevice("opttan_mr", opttanConfig.opttanMiniRContent);
      }
      return this.showTable(out);
    };

    this.checkCookie = function () {
      this.setCookie('test', true);
      if (!this.getCookieValue('test'))
        alert("In Ihrem Browser sind Cookies ausgeschaltet.\nDie Grennderung kann nur mit eingeschalteten Cookies gespeichert werden.");
    };

}




